@@ -1,5 +1,6 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
+# Part of the code for Arrow heads and text from Paul Giotta (2002)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +25,7 @@ format_name = 'SVG'
 ###End
 
 from string import join
+import re
 from math import atan2, hypot, pi
 
 from Sketch import const, Bezier, EmptyPattern, Trafo, Rotation, Translation
@@ -35,6 +37,29 @@ def csscolor(color):
 svg_joins = ('miter', 'round', 'bevel')
 svg_caps = (None, 'butt', 'round', 'square')
 
+
+# FIXME: This is the definition of a simple triangular arrowhead It is
+# copied directly from the an example in the SVG spec This is a
+# prototype only. Do not consider this an acceptable long term solution
+arrow_head_def = '''  <defs> 
+    <marker id="ArrowEnd" viewBox="0 0 10 10" refX="0" refY="5" 
+     markerUnits="strokeWidth" 
+     markerWidth="4" 
+     markerHeight="3" 
+     orient="auto"> 
+        <path d="M 0 0 L 10 5 L 0 10 z" /> 
+    </marker>
+    <marker id="ArrowStart" viewBox="0 0 10 10" refX="10" refY="5" 
+     markerUnits="strokeWidth" 
+     markerWidth="4" 
+     markerHeight="3" 
+     orient="auto"> 
+        <path d="M 10 0 L 0 5 L 10 10 z" /> 
+    </marker> </defs>
+'''
+  
+
+
 class SVGSaver:
 
     def __init__(self, file, filename, document, options):
@@ -51,6 +76,11 @@ class SVGSaver:
         self.idcount = self.idcount + 1
         return self.idcount
 
+    fontMap = {"Times"     : re.compile("Times-Roman.*"),
+               "Helvetica" : re.compile("Helvetica.*"),
+               "Courier"   : re.compile("Courier.*"),
+               }
+
     def make_style(self, properties, bounding_rect = None):
         style = []
         if properties.line_pattern is not EmptyPattern:
@@ -58,13 +88,25 @@ class SVGSaver:
                 color = properties.line_pattern.Color()
                 style.append("stroke:" + csscolor(color))
             if properties.line_dashes != ():
-                pass
+                #FIXME: This could be much more intelligent, but this
+                #quick hack will only produce one style of dashed line
+                #for the moment.
+                style.append("stroke-dasharray:" + "6,3")
             style.append('stroke-width:' + `properties.line_width`)
             if properties.line_join != const.JoinMiter:
                 style.append('stroke-linejoin:'
                              + svg_joins[properties.line_join])
             if properties.line_cap != const.CapButt:
                 style.append('stroke-linecap:' + svg_caps[properties.line_cap])
+
+            # FIXME: when arrow heads are implemented properly, change
+            # this accordingly
+            # FIXME: currently the orientation of the arrow heads is
+            # wrong.
+            if properties.line_arrow1 <> None:
+                style.append('marker-start:url(#ArrowStart)')
+            if properties.line_arrow2 <> None:
+                style.append('marker-end:url(#ArrowEnd)')
         else:
             style.append("stroke:none")
         if properties.fill_pattern is not EmptyPattern:
@@ -81,6 +123,17 @@ class SVGSaver:
         else:
             style.append("fill:none")
 
+        if properties.font is not None:
+            font = properties.font.PostScriptName()
+            size = properties.font_size
+
+            for svgfont, pattern in self.fontMap.items():
+                if pattern.match(font):
+                    font = svgfont
+                    break
+            style.append("font-family:" + font)
+            style.append("font-size:" + str(size))
+
         return join(style, '; ')
 
     def write_gradient(self, (pattern, rect), style):
@@ -153,6 +206,26 @@ class SVGSaver:
                 data.append('z')
         write('d="%s"/>\n' % join(data, ''))
 
+
+    def SimpleText(self, object):
+        style = self.make_style(object.Properties(), object.bounding_rect)
+
+        # Char glyphs are inherently upside-down in SVG (compared to
+        # sketch) since the y axis points toward the bottom of the page
+        # in SVG, but the characters are oriented toward the top of the
+        # page. This extra transform inverts them.
+        textTrafo = Trafo(1, 0, 0, -1, 0, 0)
+
+        tm = self.trafo(object.Trafo()(textTrafo)).coeff()
+
+        beginText = '<text style="%s" transform="matrix(%g %g %g %g %g %g)">\n'
+        endText = '</text >\n'
+
+        self.file.write(beginText %  (( style, ) + tm) )
+        self.file.write(object.Text())
+        self.file.write("\n")
+        self.file.write(endText)
+
     def BeginGroup(self):
         self.file.write('<g>\n')
 
@@ -160,7 +233,8 @@ class SVGSaver:
         self.file.write('</g>\n')
 
     def Save(self):
-        self.file.write('<?xml version="1.0" standalone="yes"?>\n')
+        self.file.write('<?xml version="1.0" encoding="ISO-8859-1" '
+                        'standalone="yes"?>\n')
 
         left, bottom, right, top = self.document.BoundingRect()
         width = right - left
@@ -169,6 +243,14 @@ class SVGSaver:
         self.file.write('<svg width="%g" height="%g"' % (width, height))
         #self.file.write(' transform="matrix(%g,%g,%g,%g,%g,%g)">\n' % trafo)
         self.file.write('>\n')
+
+        # Put the definition of a simple triangular arrowhead in the file,
+        # whether it is used or not
+        # FIXME: for a proper solution for arrow heads, we could walk
+        # over the object tree, collect all arrow heads actually used
+        # and write them out here.
+        self.file.write( arrow_head_def )
+
         for layer in self.document.Layers():
             if not layer.is_SpecialLayer and layer.Printable():
                 self.BeginGroup()
@@ -182,8 +264,8 @@ class SVGSaver:
                 self.BeginGroup()
                 self.save_objects(object.GetObjects())
                 self.EndGroup()
-            #elif object.is_SimpleText:
-            #    self.SimpleText(object)
+            elif object.is_SimpleText:
+                self.SimpleText(object)
             #elif object.is_Image:
             #    self.Image(object)
             elif object.is_Bezier or object.is_Rectangle or object.is_Ellipse:
