@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -312,7 +312,7 @@ class PostScriptDevice(CommonDevice):
 	self.file.write('%g %g m %g %g l s\n' % (tuple(start) + tuple(end)))
 
     def DrawLineXY(self, x1, y1, x2, y2):
-	self.file.write('%g %g m %g %g l s\n' % (x1, y1, x2, y1))
+	self.file.write('%g %g m %g %g l s\n' % (x1, y1, x2, y2))
 
     def DrawRectangle(self, start, end):
 	self.file.write('%g %g %g %g R s\n' % (tuple(start) + tuple(end)))
@@ -461,7 +461,7 @@ class PostScriptDevice(CommonDevice):
     def set_font(self, font, size):
 	spec = (font.PostScriptName(), size)
 	if self.current_font != spec:
-	    self.file.write('/%s %d sf\n' % spec)
+	    self.file.write('/%s %g sf\n' % spec)
 	    self.current_font = spec
 
     def DrawText(self, text, trafo, clip = 0, cache = None):
