@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -278,9 +278,10 @@ class SketchMainWindow(Publisher):
 		    # XXX there should be a plugin interface for this kind
 		    # of post-processing
                     if compressed == "gzip":
-                        file = os.popen('gzip -c -9 > ' + compressed_file, 'w')
+                        cmd = 'gzip -c -9 > ' + util.sh_quote(compressed_file)
                     elif compressed == "bzip2":
-                        file = os.popen('bzip2 > ' + compressed_file, 'w')
+                        cmd = 'bzip2 > ' + util.sh_quote(compressed_file)
+                    file = os.popen(cmd, 'w')
                     saver(self.document, filename, file = file)
 		else:
 		    saver(self.document, filename)
